package dev.thinhha.tunnel_server.config;

import dev.thinhha.tunnel_server.service.JwtService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.server.HandshakeInterceptor;
import org.springframework.web.util.UriComponentsBuilder;

import java.util.Map;

@Component
@RequiredArgsConstructor
@Slf4j
public class JwtWebSocketHandshakeInterceptor implements HandshakeInterceptor {

    private final JwtService jwtService;

    @Override
    public boolean beforeHandshake(ServerHttpRequest request, ServerHttpResponse response,
                                   WebSocketHandler wsHandler, Map<String, Object> attributes) throws Exception {
        try {
            String token = extractTokenFromRequest(request);
            if (token == null) {
                log.warn("No JWT token found in WebSocket handshake");
                return false;
            }

            String clientName = jwtService.extractClientName(token);
            attributes.put("clientName", clientName);
            log.info("WebSocket handshake authenticated for client: {}", clientName);
            return true;
        } catch (Exception e) {
            log.error("JWT authentication failed for WebSocket handshake", e);
            return false;
        }
    }

    @Override
    public void afterHandshake(ServerHttpRequest request, ServerHttpResponse response,
                               WebSocketHandler wsHandler, Exception exception) {
        // No action needed after handshake
    }

    private String extractTokenFromRequest(ServerHttpRequest request) {
        // Try to get token from Authorization header
        String authHeader = request.getHeaders().getFirst("Authorization");
        if (authHeader != null && authHeader.startsWith("Bearer ")) {
            return authHeader.substring(7);
        }

        // Try to get token from query parameter
        String query = request.getURI().getQuery();
        if (query != null) {
            Map<String, String> params = UriComponentsBuilder.fromUriString("?" + query)
                    .build()
                    .getQueryParams()
                    .toSingleValueMap();
            return params.get("token");
        }

        return null;
    }
}