package dev.thinhha.tunnel_server.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import dev.thinhha.tunnel_server.dto.TunnelResponseDto;
import dev.thinhha.tunnel_server.service.TunnelService;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.BinaryMessage;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.BinaryWebSocketHandler;

import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArraySet;

@Component
@Slf4j
public class WebSocketHandler extends BinaryWebSocketHandler {

    private final Set<WebSocketSession> sessions = new CopyOnWriteArraySet<>();
    private final ConcurrentHashMap<String, WebSocketSession> clientSessions = new ConcurrentHashMap<>();
    private final ObjectMapper objectMapper;
    private final TunnelService tunnelService;

    public WebSocketHandler(ObjectMapper objectMapper,
                            TunnelService tunnelService) {
        this.objectMapper = objectMapper;
        this.tunnelService = tunnelService;
    }

    @Override
    protected void handleBinaryMessage(WebSocketSession session, BinaryMessage message) throws Exception {
        try {
            byte[] payload = message.getPayload().array();
            TunnelResponseDto response = objectMapper.readValue(payload, TunnelResponseDto.class);

            if (tunnelService != null) {
                tunnelService.handleTunnelResponse(response);
            }
        } catch (Exception e) {
            log.error("Error processing binary message from session {}: {}", session.getId(), e.getMessage());
        }
    }

    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        log.error("Transport error for session {}: {}", session.getId(), exception.getMessage());
        sessions.remove(session);
        session.close();
    }


    @Override
    public void afterConnectionEstablished(final @NonNull WebSocketSession session) throws Exception {
        sessions.add(session);
        
        // Extract client name from session attributes (set by JWT interceptor)
        String clientName = (String) session.getAttributes().get("clientName");
        if (clientName != null) {
            clientSessions.put(clientName, session);
            log.info("Client {} connected with name: {}", session.getId(), clientName);
        } else {
            log.warn("Client {} connected without valid client name", session.getId());
            clientSessions.put(session.getId(), session);
        }
        
    }

    @Override
    public void afterConnectionClosed(final @NonNull WebSocketSession session, final @NonNull CloseStatus status) throws Exception {
        sessions.remove(session);
        
        // Remove from clientSessions by finding the entry with this session
        String clientName = (String) session.getAttributes().get("clientName");
        if (clientName != null) {
            clientSessions.remove(clientName);
            log.info("Client {} disconnected (name: {})", session.getId(), clientName);
        } else {
            clientSessions.remove(session.getId());
            log.info("Client {} disconnected", session.getId());
        }
    }

    @Override
    public boolean supportsPartialMessages() {
        return true;
    }

    public WebSocketSession getClientSession(String clientShortName) {
        return clientSessions.get(clientShortName);
    }

}
