package dev.thinhha.tunnel_server.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import dev.thinhha.tunnel_server.dto.TunnelRequestDto;
import dev.thinhha.tunnel_server.dto.TunnelResponseDto;
import dev.thinhha.tunnel_server.service.TunnelService;
import dev.thinhha.tunnel_server.types.TunnelRequestType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.*;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

@Component
@Slf4j
public class WebSocketTunnelHandler implements org.springframework.web.socket.WebSocketHandler {
    
    private final TunnelService tunnelService;
    private final ObjectMapper objectMapper;
    
    public WebSocketTunnelHandler(TunnelService tunnelService, ObjectMapper objectMapper) {
        this.tunnelService = tunnelService;
        this.objectMapper = objectMapper;
    }
    
    // Track WebSocket connections: clientId/path -> Map<wsConnectionId, WebSocketSession>
    private final Map<String, Map<String, WebSocketSession>> webSocketConnections = new ConcurrentHashMap<>();
    
    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        String path = session.getUri().getPath();
        String clientId = extractClientIdFromPath(path);
        String wsPath = extractWsPathFromPath(path);
        String wsConnectionId = UUID.randomUUID().toString();
        
        log.info("WebSocket connection established for client: {} path: {} wsId: {}", 
            clientId, wsPath, wsConnectionId);
        
        // Store WebSocket session
        webSocketConnections.computeIfAbsent(clientId + wsPath, k -> new ConcurrentHashMap<>())
            .put(wsConnectionId, session);
        
        // Store connection ID in session attributes
        session.getAttributes().put("wsConnectionId", wsConnectionId);
        session.getAttributes().put("clientId", clientId);
        session.getAttributes().put("wsPath", wsPath);
        
        // Send WebSocket connection request to tunnel client
        TunnelRequestDto tunnelRequest = new TunnelRequestDto();
        tunnelRequest.setRequestId(UUID.randomUUID().toString());
        tunnelRequest.setType(TunnelRequestType.WS_CONNECT);
        tunnelRequest.setPath(wsPath);
        tunnelRequest.setHeaders(extractHeaders(session));
        tunnelRequest.setClientShortName(clientId);
        tunnelRequest.setWsConnectionId(wsConnectionId);
        
        try {
            tunnelService.sendTunnelRequest(tunnelRequest);
        } catch (Exception e) {
            log.error("Failed to send WebSocket connect request to tunnel client", e);
            session.close(CloseStatus.SERVER_ERROR);
        }
    }
    
    @Override
    public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {
        if (message instanceof TextMessage textMessage) {
            handleWebSocketMessage(session, textMessage.getPayload().getBytes(), "TEXT");
        } else if (message instanceof BinaryMessage binaryMessage) {
            handleWebSocketMessage(session, binaryMessage.getPayload().array(), "BINARY");
        }
    }
    
    private void handleWebSocketMessage(WebSocketSession session, byte[] messageBody, String messageType) throws Exception {
        String wsConnectionId = (String) session.getAttributes().get("wsConnectionId");
        String clientId = (String) session.getAttributes().get("clientId");
        String wsPath = (String) session.getAttributes().get("wsPath");
        
        if (wsConnectionId == null || clientId == null) {
            log.warn("WebSocket message received without proper connection tracking");
            return;
        }
        
        // Forward message to tunnel client
        TunnelRequestDto tunnelRequest = new TunnelRequestDto();
        tunnelRequest.setRequestId(UUID.randomUUID().toString());
        tunnelRequest.setType(TunnelRequestType.WS_MESSAGE);
        tunnelRequest.setPath(wsPath);
        tunnelRequest.setBody(messageBody);
        tunnelRequest.setClientShortName(clientId);
        tunnelRequest.setWsConnectionId(wsConnectionId);
        tunnelRequest.setWsMessageType(messageType);
        
        try {
            tunnelService.sendTunnelRequest(tunnelRequest);
        } catch (Exception e) {
            log.error("Failed to forward WebSocket message to tunnel client", e);
        }
    }
    
    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
        String wsConnectionId = (String) session.getAttributes().get("wsConnectionId");
        String clientId = (String) session.getAttributes().get("clientId");
        String wsPath = (String) session.getAttributes().get("wsPath");
        
        if (wsConnectionId != null && clientId != null && wsPath != null) {
            // Remove from tracking
            Map<String, WebSocketSession> clientConnections = webSocketConnections.get(clientId + wsPath);
            if (clientConnections != null) {
                clientConnections.remove(wsConnectionId);
                if (clientConnections.isEmpty()) {
                    webSocketConnections.remove(clientId + wsPath);
                }
            }
            
            // Send close request to tunnel client
            TunnelRequestDto tunnelRequest = new TunnelRequestDto();
            tunnelRequest.setRequestId(UUID.randomUUID().toString());
            tunnelRequest.setType(TunnelRequestType.WS_CLOSE);
            tunnelRequest.setPath(wsPath);
            tunnelRequest.setClientShortName(clientId);
            tunnelRequest.setWsConnectionId(wsConnectionId);
            
            try {
                tunnelService.sendTunnelRequest(tunnelRequest);
            } catch (Exception e) {
                log.error("Failed to send WebSocket close request to tunnel client", e);
            }
            
            log.info("WebSocket connection closed for client: {} wsId: {}", clientId, wsConnectionId);
        }
    }
    
    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        log.error("WebSocket transport error: {}", exception.getMessage());
        String wsConnectionId = (String) session.getAttributes().get("wsConnectionId");
        if (wsConnectionId != null) {
            log.error("Error on WebSocket connection: {}", wsConnectionId);
        }
    }
    
    // Method to handle responses from tunnel client
    public void handleWebSocketResponse(TunnelResponseDto response) {
        String wsConnectionId = response.getWsConnectionId();
        if (wsConnectionId == null) return;
        
        // Find the WebSocket session
        WebSocketSession targetSession = findWebSocketSession(wsConnectionId);
        if (targetSession == null || !targetSession.isOpen()) {
            log.warn("WebSocket session not found or closed: {}", wsConnectionId);
            return;
        }
        
        try {
            switch (response.getType()) {
                case WS_CONNECT -> {
                    if (response.getStatusCode() != 101) {
                        // Connection failed, close the session
                        targetSession.close(CloseStatus.SERVER_ERROR);
                    }
                    // Connection successful - no action needed, session already established
                }
                case WS_MESSAGE -> {
                    // Forward message to WebSocket client
                    if ("TEXT".equals(response.getWsMessageType())) {
                        String textMessage = new String(response.getBody());
                        targetSession.sendMessage(new TextMessage(textMessage));
                    } else {
                        targetSession.sendMessage(new BinaryMessage(response.getBody()));
                    }
                }
                case WS_CLOSE -> {
                    // Close the WebSocket session
                    targetSession.close(new CloseStatus(response.getStatusCode(), ""));
                }
                case HTTP -> {
                    // HTTP responses should not be handled here
                    log.warn("Received HTTP response in WebSocket handler: {}", response.getRequestId());
                }
            }
        } catch (Exception e) {
            log.error("Error handling WebSocket response", e);
        }
    }
    
    private WebSocketSession findWebSocketSession(String wsConnectionId) {
        return webSocketConnections.values().stream()
            .flatMap(map -> map.entrySet().stream())
            .filter(entry -> entry.getKey().equals(wsConnectionId))
            .map(Map.Entry::getValue)
            .findFirst()
            .orElse(null);
    }
    
    private String extractClientIdFromPath(String path) {
        // Extract client ID from path like /ws/{clientId}/some/path
        String[] parts = path.split("/");
        return parts.length > 2 ? parts[2] : "unknown";
    }
    
    private String extractWsPathFromPath(String path) {
        // Extract WebSocket path after client ID: /ws/{clientId}/path -> /path
        String[] parts = path.split("/");
        if (parts.length > 3) {
            return "/" + String.join("/", java.util.Arrays.copyOfRange(parts, 3, parts.length));
        }
        return "/";
    }
    
    private Map<String, String> extractHeaders(WebSocketSession session) {
        Map<String, String> headers = new HashMap<>();
        session.getHandshakeHeaders().forEach((key, values) -> {
            if (!values.isEmpty()) {
                headers.put(key, values.get(0));
            }
        });
        return headers;
    }
    
    @Override
    public boolean supportsPartialMessages() {
        return false;
    }
}