package dev.thinhha.tunnel_server.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import dev.thinhha.tunnel_server.dto.ClientDto;
import dev.thinhha.tunnel_server.entity.Client;
import dev.thinhha.tunnel_server.mapper.ClientMapper;
import dev.thinhha.tunnel_server.repository.ClientRepository;
import dev.thinhha.tunnel_server.service.JwtService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Optional;

@RestController
@RequestMapping("/auth")
@Slf4j
public class AuthController {

    private final ClientRepository clientRepository;
    private final ClientMapper clientMapper;
    private final ObjectMapper objectMapper;
    private final JwtService jwtService;

    public AuthController(ClientRepository clientRepository,
                          ClientMapper clientMapper,
                          ObjectMapper objectMapper,
                          JwtService jwtService) {
        this.clientRepository = clientRepository;
        this.clientMapper = clientMapper;
        this.objectMapper = objectMapper;
        this.jwtService = jwtService;
    }

    @PostMapping("/register")
    public ResponseEntity<ClientDto> registerClient(@RequestBody ClientDto clientDto) {
        try {
            // Generate JWT token for the client
            String token = jwtService.generateToken(clientDto.getClientName());
            clientDto.setJwtToken(token);

            // Save client to database
            Client client = objectMapper.convertValue(clientDto, Client.class);
            Client savedClient = clientRepository.save(client);

            ClientDto responseDto = objectMapper.convertValue(savedClient, ClientDto.class);
            log.info("Client registered: {}", clientDto.getClientName());

            return ResponseEntity.ok(responseDto);
        } catch (Exception e) {
            log.error("Error registering client", e);
            return ResponseEntity.badRequest().build();
        }
    }

    @PostMapping("/token")
    public ResponseEntity<String> generateToken(@RequestParam String clientName) {
        try {
            // Check if client exists
            Optional<Client> clientOpt = clientRepository.findByClientName(clientName);
            if (clientOpt.isEmpty()) {
                return ResponseEntity.notFound().build();
            }

            // Generate new token
            String token = jwtService.generateToken(clientName);

            // Update client with new token
            Client client = clientOpt.get();
            client.setJwtToken(token);
            clientRepository.save(client);

            log.info("Token generated for client: {}", clientName);
            return ResponseEntity.ok(token);
        } catch (Exception e) {
            log.error("Error generating token", e);
            return ResponseEntity.badRequest().build();
        }
    }

    @GetMapping("/validate")
    public ResponseEntity<String> validateToken(@RequestParam String token) {
        try {
            String clientName = jwtService.extractClientName(token);
            return ResponseEntity.ok("Token valid for client: " + clientName);
        } catch (Exception e) {
            log.error("Token validation failed", e);
            return ResponseEntity.badRequest().body("Invalid token");
        }
    }
}