package dev.thinhha.tunnel_server.service;

import com.nimbusds.jose.*;
import com.nimbusds.jose.crypto.MACSigner;
import com.nimbusds.jose.crypto.MACVerifier;
import com.nimbusds.jwt.JWTClaimsSet;
import com.nimbusds.jwt.SignedJWT;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.time.temporal.ChronoUnit;
import java.util.Date;

@Service
@Slf4j
public class JwtService {

    @Value("${jwt.secret:your-256-bit-secret-key-here-must-be-at-least-32-chars}")
    private String jwtSecret;

    @Value("${jwt.expiration:864000000}")
    private long jwtExpiration;

    public String generateToken(String clientName) {
        try {
            JWSSigner signer = new MACSigner(jwtSecret.getBytes());

            JWTClaimsSet claimsSet = new JWTClaimsSet.Builder()
                    .subject(clientName)
                    .claim("name", clientName)
                    .issuer("tunnel-server")
                    .issueTime(new Date())
                    .expirationTime(Date.from(Instant.now().plus(jwtExpiration, ChronoUnit.SECONDS)))
                    .build();

            SignedJWT signedJWT = new SignedJWT(
                    new JWSHeader(JWSAlgorithm.HS256),
                    claimsSet);

            signedJWT.sign(signer);

            return signedJWT.serialize();
        } catch (JOSEException e) {
            log.error("Error generating JWT token", e);
            throw new RuntimeException("Failed to generate JWT token", e);
        }
    }

    public JWTClaimsSet validateToken(String token) {
        try {
            SignedJWT signedJWT = SignedJWT.parse(token);
            JWSVerifier verifier = new MACVerifier(jwtSecret.getBytes());

            if (!signedJWT.verify(verifier)) {
                throw new RuntimeException("Invalid JWT signature");
            }

            JWTClaimsSet claimsSet = signedJWT.getJWTClaimsSet();
            
            // Check expiration
            if (claimsSet.getExpirationTime().before(new Date())) {
                throw new RuntimeException("JWT token has expired");
            }

            return claimsSet;
        } catch (Exception e) {
            log.error("Error validating JWT token", e);
            throw new RuntimeException("Invalid JWT token", e);
        }
    }

    public String extractClientName(String token) {
        try {
            JWTClaimsSet claimsSet = validateToken(token);
            return claimsSet.getStringClaim("name");
        } catch (Exception e) {
            log.error("Error extracting client name from JWT", e);
            throw new RuntimeException("Failed to extract client name", e);
        }
    }
}