package dev.thinhha.tunnel_server.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import dev.thinhha.tunnel_server.config.WebSocketHandler;
import dev.thinhha.tunnel_server.config.WebSocketTunnelHandler;
import dev.thinhha.tunnel_server.dto.TunnelRequestDto;
import dev.thinhha.tunnel_server.dto.TunnelResponseDto;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.web.socket.BinaryMessage;
import org.springframework.web.socket.WebSocketSession;

import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ConcurrentHashMap;

@Service
@Slf4j
public class TunnelService {

    private final WebSocketHandler webSocketHandler;
    private final ObjectMapper objectMapper;
    private final WebSocketTunnelHandler webSocketTunnelHandler;

    private final Map<String, CompletableFuture<TunnelResponseDto>> pendingResponses = new ConcurrentHashMap<>();

    public TunnelService(@Lazy WebSocketHandler webSocketHandler,
                         ObjectMapper objectMapper,
                         @Lazy WebSocketTunnelHandler webSocketTunnelHandler) {
        this.webSocketHandler = webSocketHandler;
        this.objectMapper = objectMapper;
        this.webSocketTunnelHandler = webSocketTunnelHandler;
    }
    

    public Map<String, String> extractHeaders(HttpServletRequest request) {
        Map<String, String> headers = new HashMap<>();
        Enumeration<String> headerNames = request.getHeaderNames();

        while (headerNames.hasMoreElements()) {
            String headerName = headerNames.nextElement();
            String headerValue = request.getHeader(headerName);
            headers.put(headerName, headerValue);
        }

        return headers;
    }

    public CompletableFuture<TunnelResponseDto> sendTunnelRequest(TunnelRequestDto tunnelRequest) throws IOException {
        CompletableFuture<TunnelResponseDto> responseFuture = new CompletableFuture<>();

        // Store the future for when response comes back
        pendingResponses.put(tunnelRequest.getRequestId(), responseFuture);

        // Find the WebSocket session for the client
        WebSocketSession session = webSocketHandler.getClientSession(tunnelRequest.getClientShortName());

        if (session == null || !session.isOpen()) {
            responseFuture.completeExceptionally(new RuntimeException("Client not connected"));
            return responseFuture;
        }

        // Serialize and send the request
        byte[] messageBytes = objectMapper.writeValueAsBytes(tunnelRequest);
        BinaryMessage message = new BinaryMessage(messageBytes);

        try {
            session.sendMessage(message);
            log.info("Sent tunnel request {} to client {}", tunnelRequest.getRequestId(), tunnelRequest.getClientShortName());
        } catch (IOException e) {
            log.error("Failed to send tunnel request", e);
            pendingResponses.remove(tunnelRequest.getRequestId());
            responseFuture.completeExceptionally(e);
        }

        return responseFuture;
    }

    public void handleTunnelResponse(TunnelResponseDto response) {
        // Handle WebSocket responses separately
        if (response.getType() != null && response.getType() != dev.thinhha.tunnel_server.types.TunnelRequestType.HTTP) {
            if (webSocketTunnelHandler != null) {
                webSocketTunnelHandler.handleWebSocketResponse(response);
                log.info("Handled WebSocket response: {} type: {}", response.getRequestId(), response.getType());
            }
            return;
        }
        
        // Handle HTTP responses
        CompletableFuture<TunnelResponseDto> responseFuture = pendingResponses.remove(response.getRequestId());

        if (responseFuture != null) {
            responseFuture.complete(response);
            log.info("Completed tunnel response {}", response.getRequestId());
        } else {
            log.warn("Received response for unknown request {}", response.getRequestId());
        }
    }
}